#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <fstream>

#include <arc/common.h>
#include <arc/notify.h>

#include <fcntl.h>
#include <unistd.h>

#include <libintl.h>
#define _(A) dgettext("arclib", ((A)))


std::string GetEnv(const std::string& var) {

	char* env = getenv(var.c_str());
	return (env ? env : "");
}


std::string GlobusErrorString(globus_object_t* error) {

	std::string errstr;

	for (globus_object_t* err = error; err;
	     err = globus_error_base_get_cause(err)) {
		char* tmp = globus_object_printable_to_string(error);
		if (tmp) {
			if (!errstr.empty()) errstr += "; ";
			errstr += tmp;
			free(tmp);
		}
	}

	return errstr;
}


std::list<std::string> ReadFile(const std::string& filename) {

	notify(DEBUG) << _("Reading file") << ": " << filename << std::endl;

	std::list<std::string> lines;
	std::string line;

	std::ifstream readin(filename.c_str());
	while(getline(readin, line)) {
		// Find line delimiters - remove external whitespaces
		std::string::size_type pos1 = line.find_first_not_of(" \t");
		std::string::size_type pos2 = line.find_last_not_of(" \t");

		// Drop comments and empty lines
		if (pos1 == std::string::npos || line[pos1] == '#')
			continue;

		lines.push_back(line.substr(pos1, pos2 - pos1 + 1));
	}

	readin.close();
	return lines;
}


void LockFile(const std::string& filename) {

	notify(DEBUG) << _("Locking file") << ": " << filename << std::endl;

	std::string jobfilelock = filename + ".lock";
	int fd = open(jobfilelock.c_str(), O_WRONLY|O_CREAT|O_EXCL);
	while ((fd < 0) && (errno == EEXIST)) {
		notify(VERBOSE) << _("Waiting for file lock") << std::endl;
		usleep (10000);
		fd = open(jobfilelock.c_str(), O_WRONLY|O_CREAT|O_EXCL);
	}
}


void UnlockFile(const std::string& filename) {

	notify(DEBUG) << _("Unlocking file") << ": " << filename << std::endl;

	std::string jobfilelock = filename + ".lock";
	remove(jobfilelock.c_str());
}
